/*
*********************************************************************************************************
*                                       uC/Probe Communication
*
*                         (c) Copyright 2007-2008; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*               Knowledge of the source code may NOT be used to develop a similar product.
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                        COMMUNICATION: RS-232
*
*                                            PORT TEMPLATE
*
* Filename      : probe_rs232c.c
* Version       : V2.20
* Programmer(s) : BAN
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#include  <probe_com.h>
#include  <probe_rs232.h>

#include "altera_avalon_jtag_uart_regs.h"
#include "altera_avalon_jtag_uart.h"



/*
*********************************************************************************************************
*                                               ENABLE
*
* Note(s) : (1) See 'probe_rs232.h  ENABLE'.
*********************************************************************************************************
*/

#if (PROBE_COM_CFG_RS232_EN == DEF_ENABLED)                     /* See Note #1.                                         */


/*
*********************************************************************************************************
*                                            LOCAL DEFINES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       LOCAL GLOBAL VARIABLES
*********************************************************************************************************
*/

static  CPU_BOOLEAN  ProbeRS232_TxIntState;

/*
*********************************************************************************************************
*                                      LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                     LOCAL CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                       ProbeRS232_InitTarget()
*
* Description : Initialize the UART for Probe communication.
*
* Argument(s) : baud_rate   Intended baud rate of the RS-232.
*
* Return(s)   : none.
*
* Caller(s)   : ProbeRS232_Init().
*
* Note(s)     : (1) Though the baud rate used may vary from application to application or target to
*                   target, other communication settings are constamt.  The hardware must always be
*                   configured for the following :
*
*                   (a) No parity
*                   (b) One stop bit
*                   (c) Eight data bits.
*
*               (2) Neither receive nor transmit interrupts should be enabled by this function.
*********************************************************************************************************
*/

void  ProbeRS232_InitTarget (CPU_INT32U  baud_rate)
{
    (void)baud_rate;

    alt_ic_isr_register(UCPROBE_UART_IRQ_INTERRUPT_CONTROLLER_ID,
						UCPROBE_UART_IRQ,
    		            ProbeRS232_RxTxISRHandler,
                        NULL,
                        NULL);

    ProbeRS232_TxIntState = 0u;

}


/*
*********************************************************************************************************
*                                     ProbeRS232_RxTxISRHandler()
*
* Description : Handle receive and transmit interrupts.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : This is an ISR.
*
* Note(s)     : (1) This function may be empty if receive and transmit interrupts are handled in the
*                   separate interrupt handlers, ProbeRS232_RxISRHandler() and ProbeRS232_TxISRHandler().
*********************************************************************************************************
*/

void  ProbeRS232_RxTxISRHandler (void)
{
    CPU_INT32U  reg;
    CPU_INT32U  reg_msk;
    CPU_INT32U  rx_data;


	reg = 1; /* initial value to enter while loop */

    while (reg != 0x00) {
		reg       = IORD_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE);
	    reg_msk   = (reg << 8u) & 0x300;
	    reg      &= reg_msk;

	    if ((reg & DEF_BIT_09) > 0) {
			ProbeRS232_TxHandler();
		}

    	if ((reg & DEF_BIT_08) > 0) {
    		do {
			    rx_data = IORD_ALTERA_AVALON_JTAG_UART_DATA(UCPROBE_UART_BASE);
			    if (rx_data & 0x8000)
			        ProbeRS232_RxHandler((CPU_INT08U)rx_data);
    		} while ((rx_data & 0xFFFF0000) > 0);
		}
    }
}


/*
*********************************************************************************************************
*                                      ProbeRS232_RxISRHandler()
*
* Description : Handle receive interrupts.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : This is an ISR.
*
* Note(s)     : (1) This function may be empty if receive interrupts are handled by the combined receive/
*                   transmit interrupt handler, 'ProbeRS232_RxTxISRHandler()'.
*********************************************************************************************************
*/

void  ProbeRS232_RxISRHandler (void)
{

}


/*
*********************************************************************************************************
*                                        ProbeRS232_RxIntDis()
*
* Description : Disable receive interrupts.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) The application should call this function to stop communication.
*********************************************************************************************************
*/

void  ProbeRS232_RxIntDis (void)
{
    CPU_INT32U reg;

	reg = IORD_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE);
	reg &= ~DEF_BIT_00;
	IOWR_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE, reg);
}


/*
*********************************************************************************************************
*                                        ProbeRS232_RxIntEn()
*
* Description : Enable receive interrupts.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : Application.
*
* Note(s)     : (1) The application should call this function to start communication.
*********************************************************************************************************
*/

void  ProbeRS232_RxIntEn (void)
{
    CPU_INT32U reg;

	reg  = IORD_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE);
	reg |= DEF_BIT_00;
	IOWR_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE, reg);
}


/*
*********************************************************************************************************
*                                      ProbeRS232_TxISRHandler()
*
* Description : Handle transmit interrupts.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : This is an ISR.
*
* Note(s)     : (1) This function may be empty if transmit interrupts are handled by the combined receive/
*                   transmit interrupt handler, 'ProbeRS232_RxTxISRHandler()'.
*********************************************************************************************************
*/

void  ProbeRS232_TxISRHandler (void)
{

}


/*
*********************************************************************************************************
*                                          ProbeRS232_Tx1()
*
* Description : Transmit one byte.
*
* Argument(s) : c           The byte to transmit.
*
* Return(s)   : none.
*
* Caller(s)   : ProbeRS232_TxHandler().
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  ProbeRS232_Tx1 (CPU_INT08U c)
{
    CPU_INT32U reg;
    CPU_INT32U reg_cnt;

    if (ProbeRS232_TxIntState == 1u) {
    	reg  = IORD_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE);
    	reg |= DEF_BIT_01;
    	IOWR_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE, reg);
    }

	IOWR_ALTERA_AVALON_JTAG_UART_DATA(UCPROBE_UART_BASE, (CPU_INT32U)c);
	reg_cnt = IORD_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE) >> 16u;
	reg_cnt++;
}


/*
*********************************************************************************************************
*                                        ProbeRS232_TxIntDis()
*
* Description : Disable transmit interrupts.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : ProbeRS232_TxHandler().
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  ProbeRS232_TxIntDis (void)
{
    CPU_INT32U reg;
    CPU_SR_ALLOC();


    CPU_CRITICAL_ENTER();
	reg  = IORD_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE);
	reg &= ~DEF_BIT_01;
	IOWR_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE, reg);
    CPU_CRITICAL_EXIT();

	ProbeRS232_TxIntState = 0u;
}


/*
*********************************************************************************************************
*                                        ProbeRS232_TxIntEn()
*
* Description : Enable transmit interrupts.
*
* Argument(s) : none.
*
* Return(s)   : none.
*
* Caller(s)   : ProbeRS232_TxStart().
*
* Note(s)     : none.
*********************************************************************************************************
*/

void  ProbeRS232_TxIntEn (void)
{
    CPU_INT32U reg;
    CPU_SR_ALLOC();


    CPU_CRITICAL_ENTER();
	reg  = IORD_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE);
	reg |= DEF_BIT_01;
	IOWR_ALTERA_AVALON_JTAG_UART_CONTROL(UCPROBE_UART_BASE, reg);
	CPU_CRITICAL_EXIT();

	ProbeRS232_TxIntState = 1u;
}


/*
*********************************************************************************************************
*                                              ENABLE END
*
* Note(s) : See 'ENABLE  Note #1'.
*********************************************************************************************************
*/

#endif
